@@ -1,4 +1,4 @@
-/* $Id: xfburn-burn-data-composition-base-dialog.c 5578 2008-10-12 03:42:37Z squisher $ */
+/* $Id: xfburn-burn-data-composition-base-dialog.c 6092 2008-11-13 20:04:52Z squisher $ */
 /*
  *  Copyright (c) 2005-2007 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -528,17 +528,20 @@ thread_burn_prep_and_burn (ThreadBurnCompositionParams * params, struct burn_dri
   GtkWidget *dialog_progress = params->dialog_progress;
 
   struct burn_write_opts * burn_options;
+  struct burn_source **fifos = NULL;
   gint ret;
   int sectors[1];
 
   ret = burn_disc_add_session (disc, session, BURN_POS_END);
   if (ret == 0) {
-    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("Unable to create disc object"));
+    g_warning ("Unable to create disc object");
+    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("An error occurred in the burn backend."));
     return;
   }
 
   if (burn_track_set_source (track, params->src) != BURN_SOURCE_OK) {
-    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("Cannot attach source object to track object"));
+    g_warning ("Cannot attach source object to track object");
+    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("An error occurred in the burn backend."));
     return;
   }
   
@@ -565,7 +568,7 @@ thread_burn_prep_and_burn (ThreadBurnCompositionParams * params, struct burn_dri
     burn_write_opts_set_write_type (burn_options, BURN_WRITE_RAW, BURN_BLOCK_RAW96R);
     break;
   default:
-    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("The write mode is not supported currently"));
+    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("The write mode is not supported currently."));
     return;
   }
 
@@ -576,7 +579,16 @@ thread_burn_prep_and_burn (ThreadBurnCompositionParams * params, struct burn_dri
 
   sectors[0] = burn_disc_get_sectors (disc);
 
-  xfburn_perform_burn_write (dialog_progress, drive, params->write_mode, burn_options, disc, (params->is_fifo ? params->src : NULL), sectors);
+  if (params->is_fifo) {
+    fifos = g_new(struct burn_source *,1);
+    fifos[0] = params->src;
+  }
+
+  xfburn_perform_burn_write (dialog_progress, drive, params->write_mode, burn_options, DATA_BYTES_PER_SECTOR, disc, fifos, sectors);
+
+  if (params->is_fifo) {
+    g_free (fifos);
+  }
 
   burn_write_opts_free (burn_options);
 }
@@ -592,30 +604,22 @@ thread_burn_composition (ThreadBurnCompositionParams * params)
 
   struct burn_drive_info *drive_info = NULL;
 
-  if (!burn_initialize ()) {
-    g_critical ("Unable to initialize libburn");
-    burn_source_free (params->src);
-    g_free (params);
-    return;
-  }
-
   disc = burn_disc_create ();
   session = burn_session_create ();
   track = burn_track_create ();
 
   if (!xfburn_device_grab (params->device, &drive_info)) {
-    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("Unable to grab drive"));
+    xfburn_progress_dialog_burning_failed (XFBURN_PROGRESS_DIALOG (dialog_progress), _("Unable to grab the drive."));
   } else {
     thread_burn_prep_and_burn (params, drive_info->drive, disc, session, track);
-    burn_drive_release (drive_info->drive, params->eject ? 1 : 0);
+    xfburn_device_release (drive_info, params->eject);
   }
 
   burn_track_free (track);
   burn_session_free (session);
   burn_disc_free (disc);
-  burn_finish ();
-
   burn_source_free (params->src);
+
   g_free (params);
 }
 
@@ -639,16 +643,16 @@ cb_dialog_response (XfburnBurnDataCompositionBaseDialog * dialog, gint response_
     iso_write_opts_new (&write_opts, 0);
 
     iso_write_opts_set_iso_level (write_opts, 2);
-    iso_write_opts_set_rockridge (write_opts, TRUE);
-    iso_write_opts_set_joliet (write_opts, TRUE);
-    iso_write_opts_set_allow_deep_paths (write_opts, TRUE);
+    iso_write_opts_set_rockridge (write_opts, 1);
+    iso_write_opts_set_joliet (write_opts, 1);
+    iso_write_opts_set_allow_deep_paths (write_opts, 1);
     /* not sure if we want that
     iso_write_opts_set_omit_version_numbers (write_opts, TRUE);
     */
 
     if (iso_image_create_burn_source (priv->image, write_opts, &src) < 0) {
       /* could not create source */
-      xfce_err (_("Could not create ISO source structure"));
+      xfce_err (_("Could not create ISO source structure."));
       return;
     }
 
