@@ -1,4 +1,4 @@
-/* $Id: xfburn-data-composition.c 5403 2008-09-14 08:32:03Z squisher $ */
+/* $Id: xfburn-data-composition.c 6125 2008-11-16 19:40:19Z squisher $ */
 /*
  * Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -44,6 +44,7 @@
 #include <exo/exo.h>
 
 #include <libisofs.h>
+#include <glib/gstdio.h>
 
 #include "xfburn-data-composition.h"
 #include "xfburn-global.h"
@@ -661,7 +662,7 @@ cb_cell_file_edited (GtkCellRenderer * renderer, gchar * path, gchar * newtext,
   GtkTreePath *real_path;
 
   if (strlen (newtext) == 0) {
-    xfce_err (_("You must give a name to the file"));
+    xfce_err (_("You must give a name to the file."));
     return;
   }
     
@@ -670,7 +671,7 @@ cb_cell_file_edited (GtkCellRenderer * renderer, gchar * path, gchar * newtext,
 
   if (gtk_tree_model_get_iter (model, &iter, real_path)) {
     if (file_exists_on_same_level (model, real_path, TRUE, newtext)) {
-      xfce_err (_("A file with the same name is already present in the composition"));
+      xfce_err (_("A file with the same name is already present in the composition."));
     }
     else {
       gtk_tree_store_set (GTK_TREE_STORE (model), &iter, DATA_COMPOSITION_COLUMN_CONTENT, newtext, -1);
@@ -1009,25 +1010,25 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
   
   struct stat s;
 
-  if ((stat (path, &s) == 0)) {
-    gchar *basename = NULL;
+  if ((g_lstat (path, &s) == 0)) {
+    //gchar *basename = NULL;
     gchar *humansize = NULL;
     GtkTreeIter *parent = NULL;
     GtkTreePath *tree_path = NULL;
 
-    if (!S_ISDIR (s.st_mode) && !S_ISREG (s.st_mode)) {
+    if (!(S_ISDIR (s.st_mode) ||S_ISREG (s.st_mode) || S_ISCHR(s.st_mode) || S_ISBLK(s.st_mode) || S_ISLNK (s.st_mode))) {
       return FALSE;
     }
     
+    /* allow hidden files
     basename = g_path_get_basename (path);
     if ( (strlen (basename) > 1) && (basename[0] == '.') ) {
-      /* FIXME: is this really what we want? */
-      /* don't add hidden files/directories */
 
       g_free (basename);  
       return FALSE;
     }
     g_free (basename);
+    */
     
     xfburn_adding_progress_pulse (XFBURN_ADDING_PROGRESS (priv->progress));
     
@@ -1065,7 +1066,7 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
     
     gdk_threads_enter ();
     if (file_exists_on_same_level (model, tree_path, FALSE, name)) {
-      xfce_err (_("A file with the same name is already present in the composition"));
+      xfce_err (_("A file with the same name is already present in the composition."));
 
       gtk_tree_path_free (tree_path);
       gdk_threads_leave ();
@@ -1076,7 +1077,7 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
     gdk_threads_leave ();
     
     /* new directory */
-    if (S_ISDIR (s.st_mode)) {
+    if (S_ISDIR (s.st_mode) && !S_ISLNK (s.st_mode)) {
       GDir *dir = NULL;
       GError *error = NULL;
       const gchar *filename = NULL;
@@ -1099,6 +1100,7 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
                           DATA_COMPOSITION_COLUMN_ICON, icon_directory,
                           DATA_COMPOSITION_COLUMN_CONTENT, name,
                           DATA_COMPOSITION_COLUMN_TYPE, DATA_COMPOSITION_TYPE_DIRECTORY, 
+                          DATA_COMPOSITION_COLUMN_PATH, path,
                           DATA_COMPOSITION_COLUMN_SIZE, (guint64) 4, -1);
       xfburn_disc_usage_add_size (XFBURN_DISC_USAGE (priv->disc_usage), (guint64) 4);
       gdk_threads_leave ();
@@ -1131,7 +1133,7 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
       g_dir_close (dir);
     }
     /* new file */
-    else if (S_ISREG (s.st_mode)) {
+    else if (S_ISREG (s.st_mode) || S_ISCHR(s.st_mode) || S_ISBLK(s.st_mode) || S_ISLNK (s.st_mode)) {
 #ifdef HAVE_THUNAR_VFS
       GdkScreen *screen;
       GtkIconTheme *icon_theme;
@@ -1162,14 +1164,16 @@ thread_add_file_to_list_with_name (const gchar *name, XfburnDataComposition * dc
                           DATA_COMPOSITION_COLUMN_ICON, (G_IS_OBJECT (mime_icon) ? mime_icon : icon_file),
                           DATA_COMPOSITION_COLUMN_CONTENT, name,
                           DATA_COMPOSITION_COLUMN_HUMANSIZE, humansize,
-                          DATA_COMPOSITION_COLUMN_SIZE, (guint64) s.st_size, DATA_COMPOSITION_COLUMN_PATH, path,
+                          DATA_COMPOSITION_COLUMN_SIZE, (guint64) s.st_size, 
+                          DATA_COMPOSITION_COLUMN_PATH, path,
                           DATA_COMPOSITION_COLUMN_TYPE, DATA_COMPOSITION_TYPE_FILE, -1);
 #else
       gtk_tree_store_set (GTK_TREE_STORE (model), iter,
                           DATA_COMPOSITION_COLUMN_ICON, icon_file,
                           DATA_COMPOSITION_COLUMN_CONTENT, name,
                           DATA_COMPOSITION_COLUMN_HUMANSIZE, humansize,
-                          DATA_COMPOSITION_COLUMN_SIZE, (guint64) s.st_size, DATA_COMPOSITION_COLUMN_PATH, path,
+                          DATA_COMPOSITION_COLUMN_SIZE, (guint64) s.st_size,
+                          DATA_COMPOSITION_COLUMN_PATH, path,
                           DATA_COMPOSITION_COLUMN_TYPE, DATA_COMPOSITION_TYPE_FILE, -1);
 #endif
 
@@ -1386,7 +1390,7 @@ copy_entry_to (XfburnDataComposition *dc, GtkTreeIter *src, GtkTreeIter *dest, G
       }
     
       if (file_exists_on_same_level (model, path_level, FALSE, name)) {
-        xfce_warn (_("A file named \"%s\" already exists in this directory, the file hasn't been added"), name);
+        xfce_warn (_("A file named \"%s\" already exists in this directory, the file hasn't been added."), name);
         goto cleanup;
       }
       
@@ -1742,10 +1746,7 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
       gtk_tree_model_get (model, iter, DATA_COMPOSITION_COLUMN_TYPE, &type,
 			  DATA_COMPOSITION_COLUMN_CONTENT, &name, DATA_COMPOSITION_COLUMN_PATH, &src, -1);
 
-      if (type == DATA_COMPOSITION_TYPE_DIRECTORY)
-        r = iso_tree_add_new_dir (parent, name, &dir);
-      else
-        r = iso_tree_add_node (image, parent, src, &node);
+      r = iso_tree_add_node (image, parent, src, &node);
 
       if (r < 0) {
         if (r == ISO_NULL_POINTER)
@@ -1758,9 +1759,11 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
           g_error ("Failed adding %s as a node to the image: code %d!", src, r);
       }
 
-      /* FIXME: do we need to call iso_node_ref on node? Probably not... */
+      /* why was this called again? iso_tree_add_node seems to do
+       * a good job setting the name by itself.
       if (type != DATA_COMPOSITION_TYPE_DIRECTORY)
         iso_node_set_name (node, name);
+      */
 
       g_free (name);
       g_free (src);
@@ -1768,10 +1771,9 @@ fill_image_with_composition (GtkTreeModel *model, IsoImage *image, IsoDir * pare
       if (type == DATA_COMPOSITION_TYPE_DIRECTORY && gtk_tree_model_iter_has_child (model, iter)) {
 	GtkTreeIter child;
 
-        /* FIXME: this should not cause a sigfault... 
         if (iso_node_get_type(node) != LIBISO_DIR)
             g_error ("Expected %s to be a directory, but it isn't...\n", src);
-        */
+        dir = (IsoDir *)node;
 
 	gtk_tree_model_iter_children (model, &child, iter);
 	fill_image_with_composition (model, image, dir, &child);
