@@ -1,4 +1,4 @@
-/* $Id: xfburn-device-box.c 5403 2008-09-14 08:32:03Z squisher $ */
+/* $Id: xfburn-device-box.c 6350 2008-12-21 02:25:44Z squisher $ */
 /*
  * Copyright (c) 2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -52,6 +52,7 @@ enum {
   PROP_DISC_STATUS,
   PROP_VALID,
   PROP_BLANK_MODE,
+  PROP_ACCEPT_ONLY_CD,
 };
 
 enum {
@@ -96,6 +97,7 @@ typedef struct
   GtkWidget *combo_mode;
 
   gboolean have_asked_for_blanking;
+  gboolean accept_only_cd;
 
 #ifdef HAVE_HAL
   gulong volume_changed_handlerid;
@@ -200,6 +202,10 @@ xfburn_device_box_class_init (XfburnDeviceBoxClass * klass)
                                    g_param_spec_boolean ("blank-mode", _("Blank mode"),
                                                         _("The blank mode shows different disc status messages than regular mode"), 
                                                         FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (object_class, PROP_ACCEPT_ONLY_CD, 
+                                   g_param_spec_boolean ("accept-only-cd", _("Accept only CDs as valid discs"),
+                                                        _("Accept only CDs as valid discs"), 
+                                                        FALSE, G_PARAM_READWRITE));
 }
 
 static void
@@ -358,6 +364,9 @@ xfburn_device_box_get_property (GObject *object, guint prop_id, GValue *value, G
     case PROP_BLANK_MODE:
       g_value_set_boolean (value, priv->blank_mode);
       break;
+    case PROP_ACCEPT_ONLY_CD:
+      g_value_set_boolean (value, priv->accept_only_cd);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -392,6 +401,9 @@ xfburn_device_box_set_property (GObject *object, guint prop_id, const GValue *va
     case PROP_BLANK_MODE:
       priv->blank_mode = g_value_get_boolean (value);
       break;
+    case PROP_ACCEPT_ONLY_CD:
+      priv->accept_only_cd = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -585,7 +597,8 @@ check_disc_validity (XfburnDeviceBoxPrivate *priv)
         priv->valid_disc = device->cdrw;
         break;
       case XFBURN_PROFILE_DVDRAM:
-        priv->valid_disc = device->dvdram;
+        if (!priv->accept_only_cd)
+          priv->valid_disc = device->dvdram;
         break;
       case XFBURN_PROFILE_DVD_MINUS_R:
       case XFBURN_PROFILE_DVD_MINUS_RW_OVERWRITE:
@@ -594,7 +607,8 @@ check_disc_validity (XfburnDeviceBoxPrivate *priv)
       case XFBURN_PROFILE_DVD_PLUS_R:
       case XFBURN_PROFILE_DVD_PLUS_R_DL:
       case XFBURN_PROFILE_DVD_PLUS_RW:
-        priv->valid_disc = device->dvdr;
+        if (!priv->accept_only_cd)
+          priv->valid_disc = device->dvdr;
         break;
       default:
         g_warning ("Unknown disc profile 0x%x!", profile_no);
@@ -603,7 +617,7 @@ check_disc_validity (XfburnDeviceBoxPrivate *priv)
     }
 
     if (!priv->valid_disc) {
-        priv->status_text = _("Drive can't burn on the inserted disc!");
+        priv->status_text = _("Drive can't burn on the inserted disc");
     } else {
       priv->valid_disc = (disc_status == BURN_DISC_BLANK); 
       /* Not sure if we support appending yet, so let's disable it for the time being
@@ -612,24 +626,24 @@ check_disc_validity (XfburnDeviceBoxPrivate *priv)
       if (!priv->valid_disc) {
         switch (disc_status) {
           case BURN_DISC_EMPTY:
-            priv->status_text = _("Drive is empty!");
+            priv->status_text = _("Drive is empty");
             break;
           case BURN_DISC_APPENDABLE:
-            priv->status_text = _("Sorry, multisession is not yet supported!");
+            priv->status_text = _("Sorry, multisession is not yet supported");
             break;
           case BURN_DISC_FULL:
-            priv->status_text = _("Inserted disc is full!");
+            priv->status_text = _("Inserted disc is full");
             break;
           case BURN_DISC_UNSUITABLE:
-            priv->status_text = _("Inserted disc is unsuitable!");
+            priv->status_text = _("Inserted disc is unsuitable");
             break;
           case BURN_DISC_UNGRABBED:
-            priv->status_text = _("No access to drive (mounted?)");
+            priv->status_text = _("Cannot access drive (it might be in use)");
             break;
           default:
             /* if there is no detected device, then don't print an error message as it is expected to not have a disc status */
             if (device != NULL) {
-              priv->status_text = _("Error determining disc!");
+              priv->status_text = _("Error determining disc");
               DBG ("weird disc_status = %d", disc_status);
             }
         }
@@ -658,19 +672,19 @@ check_disc_validity (XfburnDeviceBoxPrivate *priv)
         default:
           switch (disc_status) {
             case BURN_DISC_EMPTY:
-              priv->status_text = _("Drive is empty!");
+              priv->status_text = _("Drive is empty");
               break;
             case BURN_DISC_BLANK:
-              priv->status_text = _("Inserted disc is already blank!");
+              priv->status_text = _("Inserted disc is already blank");
               break;
             case BURN_DISC_UNSUITABLE:
-              priv->status_text = _("Inserted disc is unsuitable!");
+              priv->status_text = _("Inserted disc is unsuitable");
               break;
             case BURN_DISC_UNGRABBED:
-              priv->status_text = _("No access to drive (mounted?)");
+              priv->status_text = _("Cannot access drive (it might be in use)");
               break;
             default:
-              priv->status_text = _("Error determining disc!");
+              priv->status_text = _("Error determining disc");
               DBG ("weird disc_status = %d", disc_status);
           }
       }
@@ -844,6 +858,7 @@ xfburn_device_box_new (XfburnDeviceBoxFlags flags)
                       "show-speed-selection", ((flags & SHOW_SPEED_SELECTION) != 0), 
 		      "show-mode-selection", ((flags & SHOW_MODE_SELECTION) != 0),
 		      "blank-mode", ((flags & BLANK_MODE) != 0),
+		      "accept-only-cd", ((flags & ACCEPT_ONLY_CD) != 0),
                       NULL);
   
   refresh (obj);
