@@ -1,4 +1,4 @@
-/* $Id: xfburn-preferences-dialog.c 5578 2008-10-12 03:42:37Z squisher $ */
+/* $Id: xfburn-preferences-dialog.c 6079 2008-11-12 22:10:18Z squisher $ */
 /*
  *  Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *
@@ -316,7 +316,7 @@ xfburn_preferences_dialog_init (XfburnPreferencesDialog * obj)
 
 
   /* below the device list */
-  priv->check_empty_speed_list = gtk_check_button_new_with_mnemonic (_("Show _empty speed list bug warning"));
+  priv->check_empty_speed_list = gtk_check_button_new_with_mnemonic (_("Show warning on _empty speed list"));
   gtk_box_pack_start (GTK_BOX (vbox2), priv->check_empty_speed_list, FALSE, FALSE, BORDER);
   gtk_widget_show (priv->check_empty_speed_list);
 
