@@ -1,4 +1,4 @@
-/* $Id: xfburn-transcoder-gst.c 5932 2008-11-03 06:55:02Z squisher $ */
+/* $Id: xfburn-transcoder-gst.c 6351 2008-12-21 03:09:24Z squisher $ */
 /*
  *  Copyright (c) 2005-2006 Jean-François Wauthy (pollux@xfce.org)
  *  Copyright (c) 2008      David Mohr (dmohr@mcbf.net)
@@ -42,6 +42,7 @@
 #include <libburn.h>
 
 #include <gst/gst.h>
+#include <gst/pbutils/missing-plugins.h>
 
 #include "xfburn-global.h"
 #include "xfburn-error.h"
@@ -50,10 +51,15 @@
 #include "xfburn-transcoder-gst.h"
 
 
-/* Set DEBUG_GST > 0 to be able to inspect the data just before it gets to the fd,
-                     and to get a lot more gst debugging output.
-   Set DEBUG_GST > 1 to also get a lot of gst state change messages */
-#define DEBUG_GST 1
+/* 
+ * Don't define it for no debugging output (don't set to 0, I was too lazy to clean up).
+ * Set DEBUG_GST >= 1 to be able to inspect the data just before it gets to the fd,
+ *                    and to get a lot more gst debugging output.
+ * Set DEBUG_GST >= 2 to also get notification of all bus messages.
+ * Set DEBUG_GST >= 3 to also get a lot of gst state change messages, as well as seing how fast data passes through gst
+ */
+#define DEBUG_GST 2
+
 
 /** Prototypes **/
 /* class initialization */
@@ -67,6 +73,7 @@ static void create_pipeline (XfburnTranscoderGst *trans);
 static void delete_pipeline (XfburnTranscoderGst *trans);
 static void recreate_pipeline (XfburnTranscoderGst *trans);
 static const gchar * get_name (XfburnTranscoder *trans);
+static const gchar * get_description (XfburnTranscoder *trans);
 static XfburnAudioTrack * get_audio_track (XfburnTranscoder *trans, const gchar *fn, GError **error);
 
 static struct burn_track * create_burn_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **error);
@@ -78,6 +85,9 @@ static gboolean free_burning_resources (XfburnTranscoder *trans, XfburnAudioTrac
 
 static gboolean is_initialized (XfburnTranscoder *trans, GError **error);
 
+static gboolean signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res);
+static gboolean query_and_signal_duration (XfburnTranscoderGst *trans);
+
 /* gstreamer support functions */
 static gboolean bus_call (GstBus *bus, GstMessage *msg, gpointer data);
 static void on_pad_added (GstElement *element, GstPad *pad, gboolean last, gpointer data);
@@ -130,17 +140,20 @@ typedef struct {
 #define SIGNAL_SEND_ITERATIONS 10
 /* SIGNAL_SEND_TIMEOUT_MICROS is the total time,
  * which gets divided into SIGNAL_SEND_ITERATIONS probes */
-#define SIGNAL_SEND_TIMEOUT_MICROS 1000000
+#define SIGNAL_SEND_TIMEOUT_MICROS 1500000
 
 #define STATE_CHANGE_TIMEOUT_NANOS 750000000
 
 #define XFBURN_AUDIO_TRACK_GET_GST(atrack) ((XfburnAudioTrackGst *) (atrack)->data)
 
 /* globals */
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0 && DEBUG > 0
 static guint64 total_size = 0;
 #endif
 
+static const gchar *errormsg_gst_setup = "An error occurred setting gstreamer up for transcoding";
+static const gchar *errormsg_libburn_setup = "An error occurred while setting the burning backend up";
+
 /*********************/
 /* class declaration */
 /*********************/
@@ -232,6 +245,7 @@ static void
 transcoder_interface_init (XfburnTranscoderInterface *iface, gpointer iface_data)
 {
   iface->get_name = get_name;
+  iface->get_description = get_description;
   iface->is_initialized = is_initialized;
   iface->get_audio_track = get_audio_track;
   iface->create_burn_track = create_burn_track;
@@ -250,7 +264,7 @@ create_pipeline (XfburnTranscoderGst *trans)
   XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
 
   GstElement *pipeline, *source, *decoder, *conv, *sink;
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0 && DEBUG > 0
   GstElement *id;
 #endif
   GstBus *bus;
@@ -263,7 +277,7 @@ create_pipeline (XfburnTranscoderGst *trans)
   priv->source  = source   = gst_element_factory_make ("filesrc",       "file-source");
   priv->decoder = decoder  = gst_element_factory_make ("decodebin",     "decoder");
   priv->conv    = conv     = gst_element_factory_make ("audioconvert",  "converter");
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0 && DEBUG > 0
                   id       = gst_element_factory_make ("identity",      "debugging-identity");
 #endif
   priv->sink    = sink     = gst_element_factory_make ("fdsink",        "audio-output");
@@ -271,15 +285,17 @@ create_pipeline (XfburnTranscoderGst *trans)
   //DBG ("\npipeline = %p\nsource = %p\ndecoder = %p\nconv = %p\nsink = %p", pipeline, source, decoder, conv, sink);
 
   if (!pipeline || !source || !decoder || !conv || !sink) {
+    g_warning ("A pipeline element could not be created");
     g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_GST_CREATION,
-                 _("A pipeline element could not be created"));
+                 _(errormsg_gst_setup));
     return;
   }
 
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0 && DEBUG > 0
   if (!id) {
+    g_warning ("The debug identity element could not be created");
     g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_GST_CREATION,
-                 _("The debug identity element could not be created"));
+                 _(errormsg_gst_setup));
     return;
   }
 #endif
@@ -292,7 +308,7 @@ create_pipeline (XfburnTranscoderGst *trans)
 
   gst_bin_add_many (GST_BIN (pipeline),
                     source, decoder, conv, sink, NULL);
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0 && DEBUG > 0
   gst_bin_add (GST_BIN (pipeline), id);
 #endif
 
@@ -308,18 +324,19 @@ create_pipeline (XfburnTranscoderGst *trans)
             "signed", G_TYPE_BOOLEAN, TRUE,
             NULL);
 
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0 && DEBUG > 0
   if (!gst_element_link_filtered (conv, id, caps)) {
 #else
   if (!gst_element_link_filtered (conv, sink, caps)) {
 #endif
+    g_warning ("Could not setup filtered gstreamer link");
     g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_GST_CREATION,
-                 _("Could not setup filtered gstreamer link"));
+                 _(errormsg_gst_setup));
     gst_caps_unref (caps);
     return;
   }
   gst_caps_unref (caps);
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0 && DEBUG > 0
   gst_element_link (id, sink);
   g_signal_connect (id, "handoff", G_CALLBACK (cb_handoff), id);
 #endif
@@ -346,13 +363,14 @@ delete_pipeline (XfburnTranscoderGst *trans)
   priv->pipeline = NULL;
 }
 
-static void recreate_pipeline (XfburnTranscoderGst *trans)
+static void 
+recreate_pipeline (XfburnTranscoderGst *trans)
 {
   delete_pipeline (trans);
   create_pipeline (trans);
 }
 
-#if DEBUG_GST > 1
+#if DEBUG_GST > 2
 static gchar *
 state_to_str (GstState st)
 {
@@ -372,6 +390,10 @@ state_to_str (GstState st)
 }
 #endif
 
+/*
+ * this function expects the result to be in priv->is_audio, 
+ * dbg_res is only for debugging output
+ */
 static gboolean
 signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res)
 {
@@ -391,7 +413,7 @@ signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res)
     g_usleep (SIGNAL_SEND_TIMEOUT_MICROS / SIGNAL_SEND_ITERATIONS);
     g_thread_yield ();
     if (i==9) {
-      DBG ("Noone was there to listen to the result of the identification!");
+      g_warning ("Noone was there to listen to the result of the identification!");
       /* FIXME: recreate pipeline here? This state is not the fault of gst */
       if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
         DBG ("Oops, could not reset pipeline to null");
@@ -404,29 +426,66 @@ signal_identification_done (XfburnTranscoderGst *trans, const char *dbg_res)
   g_mutex_unlock (priv->gst_mutex);
 
 #if DEBUG_GST > 0
+ #if DEBUG > 0
   DBG ("Releasing mutex (%s)", dbg_res);
+ #else
+  g_message ("Signaled identification done: %s", dbg_res);
+ #endif
 #endif
 
   return TRUE;
 }
 
 
+static gboolean
+query_and_signal_duration (XfburnTranscoderGst *trans)
+{
+  XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
+  GstFormat fmt;
+  //guint secs;
+
+  fmt = GST_FORMAT_TIME;
+  if (!gst_element_query_duration (priv->pipeline, &fmt, &priv->duration)) {
+    return FALSE;
+  }
+
+  //secs = priv->duration / 1000000000;
+  if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
+    DBG ("Failed to set state!");
+    recreate_pipeline (trans);
+  }
+
+  priv->is_audio = TRUE;
+  priv->state = XFBURN_TRANSCODER_GST_STATE_IDLE;
+
+  signal_identification_done (trans, "is audio");
+
+  return TRUE;
+}
+
+
 static gboolean
 bus_call (GstBus *bus, GstMessage *msg, gpointer data)
 {
   XfburnTranscoderGst *trans = XFBURN_TRANSCODER_GST (data);
   XfburnTranscoderGstPrivate *priv= XFBURN_TRANSCODER_GST_GET_PRIVATE (trans);
 
+#if DEBUG_GST > 1
+  DBG ("msg from %-20s: %s (%d) ", GST_OBJECT_NAME (GST_MESSAGE_SRC (msg)), GST_MESSAGE_TYPE_NAME (msg), GST_MESSAGE_TYPE (msg));
+#endif
+
   switch (GST_MESSAGE_TYPE (msg)) {
 
     case GST_MESSAGE_EOS: {
       GError *error = NULL;
       XfburnAudioTrackGst *gtrack = XFBURN_AUDIO_TRACK_GET_GST (priv->curr_track);
 
-#ifdef DEBUG_GST
+#if DEBUG_GST > 0
+  #if DEBUG > 0
       DBG ("End of stream, wrote %.0f bytes", (gfloat) total_size);
-#else
-      DBG ("End of stream");
+  #else
+      g_message ("End of stream.");
+  #endif
 #endif
 
       close (gtrack->fd_in);
@@ -443,6 +502,7 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
 
       break;
     }
+
     case GST_MESSAGE_ERROR: {
       gchar  *debug;
       GError *error;
@@ -450,13 +510,10 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
       gst_message_parse_error (msg, &error, &debug);
       g_free (debug);
 
-      /* FIXME: show this all the time? */
-      g_warning ("Gstreamer error: %s\n", error->message);
-
       switch (priv->state) {
         case XFBURN_TRANSCODER_GST_STATE_IDLE:
 #if DEBUG_GST > 0
-          DBG ("Ignoring gstreamer error while idling.");
+          g_warning ("Ignoring gstreamer error while idling: %s", error->message);
 #endif
           g_error_free (error);
           recreate_pipeline (trans);
@@ -465,6 +522,7 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
         case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
           recreate_pipeline (trans);
 
+          g_message ("gstreamer abort: %s\n", error->message);
           priv->is_audio = FALSE;
           priv->error = error;
 
@@ -473,28 +531,30 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
 
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODING:
+          g_error ("Gstreamer error while transcoding: %s", error->message);
           g_error_free (error);
-          g_error ("Gstreamer error while transcoding!");
           break;
       }
       break;
     }
+
     case GST_MESSAGE_APPLICATION: {
       if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
         DBG ("Failed to reset GST state.");
         recreate_pipeline (trans);
       }
-      signal_identification_done (trans, "no audio");
+      signal_identification_done (trans, "not audio content");
 
       break;
     }
+
     case GST_MESSAGE_STATE_CHANGED: {
       GstState state, pending, old_state;
 
       gst_message_parse_state_changed (msg, &old_state, &state, &pending);
 
 /* this is very verbose */
-#if DEBUG_GST > 1
+#if DEBUG_GST > 2
       if (pending != 0)
         DBG ("%-15s\tNew state is %s, old is %s and pending is %s", GST_OBJECT_NAME (GST_MESSAGE_SRC (msg)), state_to_str(state), state_to_str(old_state), state_to_str(pending));
       else
@@ -504,7 +564,18 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
       switch (priv->state) {
         case XFBURN_TRANSCODER_GST_STATE_IDLE:
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODING:
+          break;
+
         case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
+          if (state != GST_STATE_PAUSED)
+            break;
+          if (!query_and_signal_duration (trans)) {
+            /* this is expected to fail a couple of times,
+             * so no output unless we're in debug mode */
+#if DEBUG_GST > 1
+            DBG ("could not query stream duration (expected failure)");
+#endif
+          }
           break;
 
         case XFBURN_TRANSCODER_GST_STATE_TRANSCODE_START:
@@ -515,7 +586,7 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
             break;
           
           if (!g_mutex_trylock (priv->gst_mutex)) {
-            g_warning ("Lock held by another thread, can't signal transcoding start!");
+            g_critical ("Lock held by another thread, can't signal transcoding start!");
             break;
           } else {
 #if DEBUG_GST > 0
@@ -530,9 +601,8 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
 
       break;
     }
+
     case GST_MESSAGE_DURATION: {
-      GstFormat fmt;
-      guint secs;
 
       switch (priv->state) {
         case XFBURN_TRANSCODER_GST_STATE_IDLE:
@@ -541,32 +611,38 @@ bus_call (GstBus *bus, GstMessage *msg, gpointer data)
           break;
 
         case XFBURN_TRANSCODER_GST_STATE_IDENTIFYING:
-          fmt = GST_FORMAT_TIME;
-          if (!gst_element_query_duration (priv->pipeline, &fmt, &priv->duration)) {
+          if (!query_and_signal_duration (trans)) {
+            /* this is expected to work, because we got the duration message on the bus */
 #if DEBUG_GST > 0
-            DBG ("Could not query stream length!");
+            g_warning ("Could not query stream length!");
 #endif
             return TRUE;
           }
-
-          secs = priv->duration / 1000000000;
-          if (gst_element_set_state (priv->pipeline, GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE) {
-            DBG ("Failed to set state!");
-            recreate_pipeline (trans);
-          }
-
-          priv->is_audio = TRUE;
-          priv->state = XFBURN_TRANSCODER_GST_STATE_IDLE;
-
-          signal_identification_done (trans, "is audio");
           break;
       } /* switch of priv->state */
 
       break;
     }
+
+    case GST_MESSAGE_ELEMENT: {
+      if (gst_is_missing_plugin_message (msg)) {
+          recreate_pipeline (trans);
+
+          priv->is_audio = FALSE;
+          g_set_error (&(priv->error), XFBURN_ERROR, XFBURN_ERROR_MISSING_PLUGIN,
+                       _("%s is missing.\n"
+                         "\n"
+                         "You do not have a decoder installed to handle this file.\n"
+                         "Probably you need to look at the gst-plugins-* packages\n"
+                         "for the necessary plugins.\n"),
+                        gst_missing_plugin_message_get_description (msg)); 
+
+          signal_identification_done (trans, "missing-plugin");
+      }
+    }
     default:
-#if DEBUG_GST > 0
-      DBG ("bus call: %s (%d) ", GST_MESSAGE_TYPE_NAME (msg), GST_MESSAGE_TYPE (msg));
+#if DEBUG_GST == 1
+      DBG ("msg from %-20s: %s (%d) ", GST_OBJECT_NAME (GST_MESSAGE_SRC (msg)), GST_MESSAGE_TYPE_NAME (msg), GST_MESSAGE_TYPE (msg));
 #endif
       break;
   }
@@ -597,7 +673,7 @@ on_pad_added (GstElement *element, GstPad *pad, gboolean last, gpointer data)
   DBG ("linking pads");
 #endif
 
-  // check media type
+  // check disc type
   caps = gst_pad_get_caps (pad);
   str = gst_caps_get_structure (caps, 0);
   if (!g_strrstr (gst_structure_get_name (str), "audio")) {
@@ -639,10 +715,26 @@ on_pad_added (GstElement *element, GstPad *pad, gboolean last, gpointer data)
 static const gchar * 
 get_name (XfburnTranscoder *trans)
 {
-  return "gstreamer";
+  /* Note to translators: you can probably keep this as gstreamer,
+   * unless you have a good reason to call it by another name that
+   * the user would understand better */
+  return _("gstreamer");
 }
 
-#ifdef DEBUG_GST
+static const gchar * 
+get_description (XfburnTranscoder *trans)
+{
+  return _("The gstreamer transcoder uses the gstreamer\n"
+           "library for creating audio compositions.\n"
+           "\n"
+           "Essentially all audio files should be supported\n"
+           "given that the correct plugins are installed.\n"
+           "If an audio file is not recognized, make sure\n"
+           "that you have the 'good','bad', and 'ugly'\n"
+           "gstreamer plugin packages installed.");
+}
+
+#if DEBUG_GST > 0 && DEBUG > 0
 
 /* this function can inspect the data just before it is passed on
    to the fd for processing by libburn */
@@ -650,8 +742,16 @@ static void
 cb_handoff (GstElement *element, GstBuffer *buffer, gpointer data)
 {
   guint size = GST_BUFFER_SIZE (buffer);
+#if DEBUG_GST > 2
+  static int i = 0;
+  const int step = 30;
+#endif
 
   total_size += size;
+#if DEBUG_GST > 2
+  if (++i % step == 0)
+    DBG ("gstreamer just processed ~%6d bytes (%8.0f bytes total).", size*step, (float)total_size);
+#endif
 }
 
 #endif
@@ -676,12 +776,9 @@ get_audio_track (XfburnTranscoder *trans, const gchar *fn, GError **error)
   priv->state = XFBURN_TRANSCODER_GST_STATE_IDENTIFYING;
   g_object_set (G_OBJECT (priv->source), "location", fn, NULL);
   if (gst_element_set_state (priv->pipeline, GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE) {
-    g_warning ("Supposedly failed to change gstreamer state, ignoring it.");
-    /*
-    g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_STATE,
-                 _("Failed to change state!"));
-    return NULL;
-    */
+#if DEBUG_GST > 0
+    g_message ("Supposedly failed to change gstreamer state, ignoring it as usually it does it anyways.");
+#endif
   }
 
   g_get_current_time (&tv);
@@ -702,7 +799,7 @@ get_audio_track (XfburnTranscoder *trans, const gchar *fn, GError **error)
 
   if (!priv->is_audio) {
     g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_NOT_AUDIO_FORMAT,
-                 _("%s\n\tis not an audio file:\n\n%s"), fn, priv->error->message);
+                 _("%s\n\nis not an audio file:\n\n%s"), fn, priv->error->message);
     g_error_free (priv->error);
     priv->error = NULL;
     return NULL;
@@ -753,8 +850,9 @@ create_burn_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **e
 
   atrack->src = burn_fd_source_new (atrack->fd, -1 , gtrack->size);
   if (atrack->src == NULL) {
+    g_warning ("Could not create burn_source from %s.", atrack->inputfile);
     g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_BURN_SOURCE,
-                 _("Could not create burn_source from %s!"), atrack->inputfile);
+                 _(errormsg_libburn_setup));
     XFBURN_AUDIO_TRACK_DELETE_DATA (atrack);
     atrack->fd = -1;
     close (pipe_fd[0]);  close (pipe_fd[1]);
@@ -771,8 +869,9 @@ create_burn_track (XfburnTranscoder *trans, XfburnAudioTrack *atrack, GError **e
   track = burn_track_create ();
   
   if (burn_track_set_source (track, atrack->src) != BURN_SOURCE_OK) {
+    g_warning ("Could not add source to track %s.", atrack->inputfile);
     g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_BURN_SOURCE,
-                 _("Could not add source to track %s!"), atrack->inputfile);
+                 _(errormsg_libburn_setup));
     XFBURN_AUDIO_TRACK_DELETE_DATA (atrack);
     burn_source_free (atrack->src);
     atrack->fd = -1;
@@ -849,6 +948,7 @@ transcode_next_track (XfburnTranscoderGst *trans, GError **error)
 #endif
 
   if (gst_element_set_state (priv->pipeline, GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE) {
+    g_warning ("Failed to change songs.");
     g_set_error (error, XFBURN_ERROR, XFBURN_ERROR_GST_STATE,
                  _("Failed to change songs while transcoding"));
     return FALSE;
